
/*******************************************************************************
*
* File check1.c
*
* Copyright (C) 2014 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Binary search trees.
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "utils.h"
#include "dmlist.h"

static int ndat=0,tid=0;
static double *data;
static double arnold[2]={0.1234,0.5678};


static double random(void)
{
   double x,y;

   x=2.0*arnold[0]+arnold[1];
   y=arnold[0]+arnold[1];

   if (x>=2.0)
      x-=2.0;
   else if (x>=1.0)
      x-=1.0;

   if (y>=1.0)
      y-=1.0;

   arnold[0]=x;
   arnold[1]=y;

   return x;
}


static int rank(int oid1,int oid2,void *context)
{
   int o1,o2;
   double *a;

   a=(double*)(context);

   o1=(int)(a[oid1]);
   o2=(int)(a[oid2]);

   if (o1<o2)
      return -1;
   else if (o1>o2)
      return 1;
   else
      return 0;
}


static int is_equal(int oid1,int oid2,void *context)
{
   double *a;

   a=(double*)(context);

   return (a[oid1]==a[oid2]);
}


int main(void)
{
   int n,i,ie,ifail;

   printf("\n");
   printf("Binary search trees\n");
   printf("-------------------\n\n");

   while(1)
   {
      if (tid>1)
      {
         free(data);
         free_aatree(1);
      }

      printf("Specify array size: ");
      (void)(scanf("%d",&ndat));
      error(ndat<1,1,"main [check1.c]","Array size must be positive");

      data=malloc((ndat+1)*sizeof(*data));
      error(data==NULL,1,"main [check1.c]","Unable to allocate array");

      tid=init_aatree((void*)(data),rank,is_equal);
      n=0;

      for (i=0;i<ndat;i++)
      {
         data[n]=floor((double)(ndat)*random())/4.0;
         n+=insert_aatree(n,tid);
      }

      printf("Inserted %d data items in search tree no %d\n",n,tid);
      printf("Maximal search path length = %d\n",check_aatree(tid));

      ie=0;
      ifail=0;

      for (i=0;i<n;i++)
      {
         data[n]=data[i];
         ie|=(insert_aatree(n,tid)!=0);
      }

      error(ie!=0,1,"main [check1.c]",
            "Inserted the same object twice");

      for (i=0;i<n;i++)
      {
         data[n]=data[i];
         ie|=(i!=search_aatree(n,tid,&ifail));
         ie|=(ifail!=0);
      }

      data[n]=-1.0;
      ie|=(n!=search_aatree(n,tid,&ifail));
      ie|=(ifail!=1);

      error(ie!=0,1,"main [check1.c]","Inserted data not found again");
      printf("No errors detected\n\n");
   }

   exit(0);
}
