
/*******************************************************************************
*
* File check1.c
*
* Copyright (C) 2014 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Consistency checks on the branch list generated by branches().
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "utils.h"
#include "trees.h"
#include "rvt.h"


static theory_t theory(char *name)
{
   if (strcmp("PHI4",name)==0)
      return PHI4;
   else if (strcmp("YM",name)==0)
      return YM;
   else if (strcmp("QCD",name)==0)
      return QCD;
   else if (strcmp("LYM",name)==0)
      return LYM;
   else if (strcmp("LQCD",name)==0)
      return LQCD;
   else
      error(1,1,"theory [check1.c]","Unknown theory");

   return THEORIES;
}


static void rvt_branch(branch_t *br1,branch_t *br2)
{
   int nv,i;

   error((*br1).nv>(*br2).nv,1,"rvt_branch [check1.c]",
         "Branch data arrays are too short");

   nv=(*br1).nv;
   (*br2).nv=nv;

   for (i=0;i<nv;i++)
   {
      (*br2).v[i][0]=(*br1).v[nv-1-i][0];
      (*br2).v[i][1]=(*br1).v[nv-1-i][1];

      if (i<(nv-1))
         (*br2).l[i]=rvt[(int)((*br1).l[nv-2-i])];
   }
}


static int check_branches(theory_t th,int order,int nbr)
{
   int nv,nvm,ibr;
   int i,ord,ie;
   ltype_t l,ln;
   vertex_t *v;
   branch_t *br,*brv;
   vlist_t vl;

   nvm=0;

   for (ibr=0;ibr<nbr;ibr++)
   {
      nv=get_nv(th,ibr);

      if (nv>nvm)
         nvm=nv;
   }

   br=alloc_branch(nvm);
   brv=alloc_branch(nvm);
   vl=vertices(th,order);
   ie=0;

   for (ibr=0;ibr<nbr;ibr++)
   {
      (*br).nv=nvm;
      get_branch(th,ibr,br);

      for (i=0;i<(*br).nv;i++)
      {
         ie|=(((*br).v[i][0]>vl.order)||
              ((*br).v[i][1]>=vl.nv[(*br).v[i][0]]));
      }

      if (ie!=0)
         break;
      ord=0;

      for (i=0;i<(*br).nv;i++)
      {
         v=vl.v[(*br).v[i][0]]+(*br).v[i][1];
         ord+=(*v).order;
         ie|=((*v).type==EXTERNAL);
         ie|=(((*v).type==SOURCE)&&(i!=0)&&(i!=((*br).nv-1)));

         if (i<((*br).nv-1))
         {
            l=(*br).l[i];
            ie|=(l==LTYPES);
         }
      }

      ie|=(get_order(th,ibr)!=ord);
      ie|=(ord>order);
      if (ie!=0)
         break;

      if ((*br).nv>1)
      {
         for (i=0;i<(*br).nv;i++)
         {
            v=vl.v[(*br).v[i][0]]+(*br).v[i][1];

            if (i<((*br).nv-1))
            {
               l=(*br).l[i];

               if (i==0)
                  ie|=((*v).nl[(int)(l)]<1);
               else
               {
                  ln=(*br).l[i-1];
                  ln=rvt[(int)(ln)];

                  if (l!=ln)
                     ie|=((*v).nl[(int)(l)]<1);
                  else
                     ie|=((*v).nl[(int)(l)]<2);
               }
            }
            else
            {
               ln=(*br).l[i-1];
               ln=rvt[(int)(ln)];
               ie|=((*v).nl[(int)(ln)]<1);
            }
         }
      }

      (*brv).nv=nvm;
      rvt_branch(br,brv);

      ie|=(find_branch(th,br)!=ibr);
      ie|=(find_branch(th,brv)!=inverse_branch(th,ibr));
      if (ie!=0)
         break;
   }

   error(ie!=0,1,"check_branches [check1.c]","Unexpected branch data");

   free_branch(br);
   free_branch(brv);

   return nvm;
}


static int nosrc(theory_t th,int order,int nbr,int nvm)
{
   int nsb,ibr,n;
   vertex_t *v1,*v2;
   branch_t *br;
   vlist_t vl;

   br=alloc_branch(nvm);
   vl=vertices(th,order);
   nsb=0;

   for (ibr=0;ibr<nbr;ibr++)
   {
      (*br).nv=nvm;
      get_branch(th,ibr,br);

      n=(*br).nv-1;
      v1=vl.v[(*br).v[0][0]]+(*br).v[0][1];
      v2=vl.v[(*br).v[n][0]]+(*br).v[n][1];

      if (((*v1).type!=SOURCE)&&((*v2).type!=SOURCE))
      {
         if (ibr==inverse_branch(th,ibr))
            nsb+=2;
         else
            nsb+=1;
      }
   }

   error((nsb%2)!=0,1,"nosrc [check1.c]","Unexpected branch count");
   free_branch(br);

   return nsb/2;
}


int main(void)
{
   int order,nbr,nvm,nsb;
   char name[5];
   theory_t th;

   printf("\n");
   printf("List of branches\n");
   printf("----------------\n\n");

   while (1)
   {
      printf("Specify theory and maximal order: ");
      (void)scanf("%4s %d",name,&order);

      th=theory(name);
      nbr=branches(th,order);
      printf("\n");
      printf("Total number of branches = %d\n",nbr);

      nvm=check_branches(th,order,nbr);
      printf("Maximal number of vertices in the branches = %d\n",nvm);

      nsb=nosrc(th,order,nbr,nvm);
      printf("Number of branches w/o SOURCE vertices = %d\n\n",nsb);
   }

   exit(0);
}
